%{
#include <stdio.h>
#include <stdlib.h>
#include "store.h"
#include "jit.h"

extern int yylineno;
char tmp[128];
char tmp2[128];

void yyerror(const char *str)
{
	fprintf(stderr, "!! [%i] %s\n", yylineno, str);
}

int yywrap()
{
	return 1;
}

int main()
{
	store_init();

	yyparse();
	return 0;
}

%}

%token SYM_IDENT SYM_IDENT_PP SYM_IDENT_MM SYM_LABEL
%token SYM_ASSIGN SYM_NUMBER
%token SYM_OP_PLUS SYM_OP_MINUS SYM_OP_MULT SYM_OP_DIV
%token SYM_BREAK
%token SYM_BRACKETB SYM_BRACKETE SYM_EXCLMARK SYM_COLON
%token SYM_METHOD

%left SYM_OP_PLUS SYM_OP_MINUS
%left SYM_EXCLMARK

%nonassoc SYM_ASSIGN

%%

commands: /*empty */
        | commands command
        ;

command:
        assignment break
        |
        outputident break
        ;

assignment:
            ident assign term
			{
				printf("ASSIGN: %s=%s\n", $1, $3);
				printf("Jitter: %s\n", jit_execute());
				if(jit_execute())
					store_set((char*)$1, jit_execute());
				else
					store_set((char*)$1, (char*)$3);
				jit_enable(0);
			}
            ;

term:
    term op_minus subident
    {
        printf("TERM: %s - %s\n", $1, $3);
        $$=($1 ? atoi((char*)$1) : 0) - ($3 ? atoi((char*)$3) : 0);
		snprintf(tmp, sizeof(tmp), "%i", $$);
		$$=(YYSTYPE)tmp;
		printf("=> %s\n", $$);
    }
    |
    term op_plus subident
	{
		printf("TERM: %s + %s\n", $1, $3);
        $$=($1 ? atoi((char*)$1) : 0) + ($3 ? atoi((char*)$3) : 0);
		snprintf(tmp, sizeof(tmp), "%i", $$);
		$$=(YYSTYPE)tmp;
		printf("=> %s\n", $$);
	}
    |
    term op_mult subident
	{
		printf("TERM: %s * %s\n", $1, $3);
        $$=($1 ? atoi((char*)$1) : 0) * ($3 ? atoi((char*)$3) : 0);
		snprintf(tmp, sizeof(tmp), "%i", $$);
		$$=(YYSTYPE)tmp;
		printf("=> %s\n", $$);
	}
    |
    term op_div subident
	{
		printf("TERM: %s / %s\n", $1, $3);
		if($3 && atoi((char*)$3))
		{
    	    $$=($1 ? atoi((char*)$1) : 0) / ($3 ? atoi((char*)$3) : 0);
			snprintf(tmp, sizeof(tmp), "%i", $$);
		}
		else snprintf(tmp, sizeof(tmp), "NaN");
		$$=(YYSTYPE)tmp;
		printf("=> %s\n", $$);
	}
    |
	SYM_OP_MINUS subident
	{
		printf("[minus] %s %s!\n", $1, $2);
		$$=-atoi((char*)$2);
		snprintf(tmp, sizeof(tmp), "%i", $$);
		$$=(YYSTYPE)tmp;
	}
	|
    subident
    ;

op_minus:
	SYM_OP_MINUS
	{
		jit_compile("-");
	}
	;
op_plus:
	SYM_OP_PLUS
	{
		jit_compile("+");
	}
	;
op_mult:
	SYM_OP_MULT
	{
		jit_compile("*");
	}
	;
op_div:
	SYM_OP_DIV
	{
		jit_compile("/");
	}
	;

subident:
         bracketb term brackete
		 {
		 	printf("brackets: %s %s %s\n", $1, $2, $3);
			$$=$2;
		 }
         |
		 outputident method outputident bracketb brackete
		 {
		 	printf("METHOD CALL: %s %s %s %s %s\n", $1, $2, $3, $4, $5);
			$$=(YYSTYPE)store_method((char*)$2, (char*)$4);
		 }
		 |
         outputident exclmark
		 {
		 	printf("re-evaluate %s %s\n", $1, $2);
			$$=(YYSTYPE)store_get((char*)$2);
		 }
		 |
		 outputident_minus_minus
		 |
		 outputident_plus_plus
		 |
		 outputident
         |
         number
         ;

break:
       SYM_BREAK
       {
       printf("***\n");
       }
       ;

ident:
       real_ident
	   |
	   real_ident label colon
	   |
	   real_ident label
	   |
	   real_ident colon
	   ;

real_ident:
          SYM_IDENT
          {
          printf("(input-ident %s)\n", $1);
          };

outputident_plus_plus:
          SYM_IDENT_PP
          {
		  snprintf(tmp, sizeof(tmp), "%s", (char*)$1);
		  tmp[strlen(tmp) - 1] = '\0';
		  tmp[strlen(tmp) - 1] = '\0';
          printf("(output-ident++ %s)\n", tmp);
		  jit_compile((char*)$1);
          $$=(YYSTYPE)store_get(tmp);
		  $$=($$ ? atoi((char*)$$) + 1 : 1);
		  snprintf(tmp2, sizeof(tmp2), "%i", $$);
		  store_set(tmp, tmp2);
		  $$=(YYSTYPE)tmp2;
          printf("-> %s\n", $$);
          };

outputident_minus_minus:
          SYM_IDENT_MM
          {
		  snprintf(tmp, sizeof(tmp), "%s", (char*)$1);
		  tmp[strlen(tmp) - 1] = '\0';
		  tmp[strlen(tmp) - 1] = '\0';
          printf("(output-ident-- %s)\n", $1);
		  jit_compile((char*)$1);
          $$=(YYSTYPE)store_get(tmp);
		  $$=($$ ? atoi((char*)$$) - 1 : -1);
		  snprintf(tmp2, sizeof(tmp2), "%i", $$);
		  store_set(tmp, tmp2);
		  $$=(YYSTYPE)tmp2;
          printf("-> %s\n", $$);
          };

outputident:
          SYM_IDENT
          {
          printf("(output-ident %s)\n", $1);
		  jit_compile((char*)$1);
          $$=(YYSTYPE)store_get((char*)$1);
          printf("-> %s\n", $$);
          };

number:
          SYM_NUMBER
          {
          printf("(number)\n");
		  jit_compile((char*)$1);
          $$=$1;
          };

assign:
          SYM_ASSIGN
          {
          printf("(assign)\n");
		  jit_reset();
          };

bracketb:
          SYM_BRACKETB
          {
          printf("bracket:(\n");
		  jit_compile("(");
          }
		  ;

brackete:
          SYM_BRACKETE
          {
          printf("bracket:) (%s %s)\n", $1, $$);
		  jit_compile(")");
          }
		  ;

exclmark:
			SYM_EXCLMARK
			{
			printf("!!!!!!!!!!!! (%s)\n", $1);
			}
			;

label:
			SYM_LABEL
			{
			printf("i-f label: %s\n", $1);
			}
			;

method:
			SYM_METHOD
			{
			printf("METHOD() %s\n", $1);
			jit_compile(".");
			}
			;
colon:
			SYM_COLON
			{
			printf("colon\n");
			jit_enable(1);
			}
			;

